#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library
import codecs
import chardet
from urllib import parse

# import interfaces
from pyams_scheduler.interfaces.url import IURLCallerTask

# import packages
from pyams_scheduler.task import Task
from pyams_utils.html import html_to_text
from pyams_utils.protocol.http import HTTPClient
from pyramid.exceptions import ConfigurationError
from zope.interface import implementer
from zope.schema.fieldproperty import FieldProperty


@implementer(IURLCallerTask)
class URLCallerTask(Task):
    """URL caller task"""

    url = FieldProperty(IURLCallerTask['url'])
    username = FieldProperty(IURLCallerTask['username'])
    password = FieldProperty(IURLCallerTask['password'])
    proxy_server = FieldProperty(IURLCallerTask['proxy_server'])
    proxy_port = FieldProperty(IURLCallerTask['proxy_port'])
    remote_dns = FieldProperty(IURLCallerTask['remote_dns'])
    proxy_username = FieldProperty(IURLCallerTask['proxy_username'])
    proxy_password = FieldProperty(IURLCallerTask['proxy_password'])
    connection_timeout = FieldProperty(IURLCallerTask['connection_timeout'])

    def run(self, report):
        parser = parse.urlparse(self.url)
        if not parser.netloc:
            raise ConfigurationError("Missing hostname. Task aborted.")
        if self.proxy_server and not self.proxy_port:
            raise ConfigurationError("Proxy server defined without proxy port. Task aborted.")
        params = parser.query and dict([part.split('=') for part in parser.query.split('&')]) or {}
        credentials = (self.username, self.password) if self.username else ()
        proxy = (self.proxy_server, self.proxy_port) if self.proxy_server else ()
        proxy_auth = (self.proxy_username, self.proxy_password) if self.proxy_username else ()
        client = HTTPClient('GET', parser.scheme, parser.netloc, parser.path, params=params,
                            credentials=credentials, proxy=proxy, rdns=self.remote_dns, proxy_auth=proxy_auth,
                            timeout=self.connection_timeout)
        response, content = client.get_response()
        if response.status == 200:
            message = '\n'.join(['{0}={1}'.format(k, v) for k, v in sorted(response.items())])
            if isinstance(message, bytes):
                message = message.decode()
            report.write(message)
            report.write('\n\n')
            content_type = response.get('Content-Type', 'text/plain')
            if content_type.startswith('text/html'):
                content = html_to_text(content)
            if 'charset=' in content_type.lower():
                charset = content_type.split('=', 1)[1]
            else:
                charset = chardet.detect(content).get('encoding') or 'utf-8'
            report.write(codecs.decode(content, charset))
